@@ -1,12 +1,12 @@
 ;;; dired-preview.el --- Automatically preview file at point in Dired -*- lexical-binding: t -*-
 
-;; Copyright (C) 2023-2024  Free Software Foundation, Inc.
+;; Copyright (C) 2023-2025  Free Software Foundation, Inc.
 
 ;; Author: Protesilaos Stavrou <info@protesilaos.com>
 ;; Maintainer: Protesilaos Stavrou <info@protesilaos.com>
 ;; URL: https://github.com/protesilaos/dired-preview
-;; Version: 0.3.0
-;; Package-Requires: ((emacs "27.1"))
+;; Version: 0.4.0
+;; Package-Requires: ((emacs "28.1"))
 ;; Keywords: files, convenience
 
 ;; This file is NOT part of GNU Emacs.
@@ -103,13 +103,37 @@ user option."
   :group 'dired-preview
   :type 'natnum)
 
+(defcustom dired-preview-kill-buffers-method (cons 'buffer-number 10)
+  "Determine whether to periodically kill preview buffers while in Dired.
+When the value is nil, do not kill any preview buffer.
+
+When the value is a cons cell of the form (SYMBOL . NATURAL-NUMBER),
+check if symbol is one of the following to derive the meaning of its
+NATURAL-NUMBER.
+
+- `buffer-number' means to kill the number of preview buffers that
+  exceed the NATURAL-NUMBER.
+
+- `combined-size' means to kill buffers until their combined size does
+  not exceed the NATURAL-NUMBER.
+
+Whatever the SYMBOL, buffers are killed from oldest to newest.
+
+Buffers are always killed when exiting Dired."
+  :type '(choice
+          (cons (choice (const :tag "Maximum number of buffers" buffer-number)
+                        (const :tag "Maximum cumulative buffer size" combined-size))
+                natnum)
+          (const :tag "Do not kill any preview buffers" nil))
+  :package-version '(dired-preview . "0.4.0")
+  :group 'dired-preview)
+
 (define-obsolete-variable-alias
   'dired-preview-display-action-alist-function
   'dired-preview-display-action-alist
   "0.3.0")
 
-(defcustom dired-preview-display-action-alist
-  #'dired-preview-display-action-alist-dwim
+(defcustom dired-preview-display-action-alist #'dired-preview-display-action-alist-dwim
   "The `display-buffer' action alist for the preview window.
 This is the same data that is passed to `display-buffer-alist'.
 Read Info node `(elisp) Displaying Buffers'.  As such, it is
@@ -123,10 +147,31 @@ Example of a valid value:
       (preserve-size . (t . t)))
 
 The value may also be a function, which returns a `display-buffer'
-action alist.  See `dired-preview-display-action-alist-dwim' for the
-implementation details."
+action alist.  See `dired-preview-display-action-alist-dwim' (the
+default value of this variable) for the implementation details.  A
+simpler alternative is the function `dired-preview-display-action-alist-below'.
+
+The `dired-preview-display-action-alist-dwim' will display the preview
+window either at the right hand side or the bottom of the frame,
+depending on the available space.  It will also try to resize the window
+accordingly.
+
+Whereas the `dired-preview-display-action-alist-below' has a more simple
+behaviour of always displaying the preview window below the currently
+selected window and always setting the preview window's height to 0.3
+times the height of the frame."
   :group 'dired-preview
-  :type 'function)
+  :package-version '(dired-preview . "0.4.0")
+  :type `(choice
+          (alist :key-type
+                 (choice :tag "Condition"
+                         regexp
+                         (function :tag "Matcher function"))
+                 :value-type ,display-buffer--action-custom-type)
+          (function :tag "Open to the side or bottom" dired-preview-display-action-alist-dwim)
+          (function :tag "Open below the selected window" dired-preview-display-action-alist-below)
+          (function :tag "Custom function like `dired-preview-display-action-alist-dwim'"))
+  :risky t)
 
 (defcustom dired-preview-delay 0.7
   "Time in seconds to wait before previewing."
@@ -138,6 +183,12 @@ implementation details."
   :group 'dired-preview
   :type 'natnum)
 
+(defcustom dired-preview-buffer-name-indicator "[P]"
+  "String to prepend to the name of preview buffers."
+  :type 'string
+  :package-version '(dired-preview . "0.4.0")
+  :group 'dired-preview)
+
 (defvar dired-preview--buffers nil
   "List with buffers of previewed files.")
 
@@ -148,45 +199,67 @@ implementation details."
   "Return buffers that show previews."
   (seq-filter #'buffer-live-p dired-preview--buffers))
 
-;; TODO 2023-07-07: This can become a user option, but let's keep it
-;; simple for now.  We need to be sure this is always doing the right
-;; thing.
-(defvar dired-preview--buffers-threshold (* 1000 1024)
-  "Maximum cumulative buffer size of previews.
-When the accumulated preview buffers exceed this number and
-`dired-preview--kill-buffers' is called, it will kill buffers
-until it drops below this number.")
-
-(defun dired-preview--get-buffer-cumulative-size ()
-  "Return cumulative buffer size of `dired-preview--get-buffers'."
+(defun dired-preview--get-buffer-cumulative-size (buffers)
+  "Return cumulative size of BUFFERS."
   (let ((size 0))
-    (mapc
-     (lambda (buffer)
-       (setq size (+ (buffer-size buffer) size)))
-     (dired-preview--get-buffers))
+    (dolist (buffer buffers)
+      (setq size (+ (buffer-size buffer) size)))
     size))
 
-(defun dired-preview--kill-buffers ()
-  "Kill preview buffers up to `dired-preview--buffers-threshold'."
-  (let ((buffers (nreverse (dired-preview--get-buffers))))
-    (catch 'stop
-      (mapc
-       (lambda (buffer)
-         (when (and (>= (dired-preview--get-buffer-cumulative-size) dired-preview--buffers-threshold)
-                    (not (eq buffer (current-buffer))))
-           (ignore-errors (kill-buffer-if-not-modified buffer))
-           (setq buffers (delq buffer buffers)))
-         (throw 'stop t))
-       buffers))
-    (setq dired-preview--buffers (delq nil (nreverse buffers)))))
+(defun dired-preview--kill-buffers-by-size (buffers max-combined-size)
+  "Kill BUFFERS to not exceed MAX-COMBINED-SIZE."
+  (catch 'enough
+    (dolist (buffer buffers)
+      (if (>= (dired-preview--get-buffer-cumulative-size buffers) max-combined-size)
+          (if (eq buffer (current-buffer))
+              (setq buffers (delq buffer buffers))
+            (ignore-errors (kill-buffer-if-not-modified buffer)))
+        (throw 'enough t))))
+  (setq dired-preview--buffers (delq nil (nreverse buffers))))
+
+(defun dired-preview--kill-buffers-by-length (buffers max-length)
+  "Kill BUFFERS up to MAX-LENGTH."
+  (let ((length (length buffers)))
+    (catch 'enough
+      (dolist (buffer buffers)
+        (if (>= length max-length)
+            (progn
+              (if (eq buffer (current-buffer))
+                  (setq buffers (delq buffer buffers))
+                (ignore-errors (kill-buffer-if-not-modified buffer)))
+              (setq length (1- length)))
+          (throw 'enough t)))))
+  (setq dired-preview--buffers (delq nil (nreverse buffers))))
+
+(defun dired-preview--kill-buffers-unconditionally (buffers)
+  "Kill all BUFFERS except the current one."
+  (dolist (buffer buffers)
+    (when (not (eq buffer (current-buffer)))
+      (ignore-errors (kill-buffer-if-not-modified buffer)))
+    (setq buffers (delq buffer buffers)))
+  (setq dired-preview--buffers (delq nil (nreverse buffers))))
+
+(defun dired-preview--kill-buffers (&optional kill-all)
+  "Implement `dired-preview-kill-buffers-method'.
+With optional KILL-ALL, kill all buffers regardless of the
+aforementioned user option."
+  (when-let* ((buffers (nreverse (dired-preview--get-buffers))))
+    (cond
+     (kill-all
+      (dired-preview--kill-buffers-unconditionally buffers))
+     (dired-preview-kill-buffers-method
+      (pcase-let ((`(,method . ,number) dired-preview-kill-buffers-method))
+        (pcase method
+          ('combined-size (dired-preview--kill-buffers-by-size buffers number))
+          ('buffer-number (dired-preview--kill-buffers-by-length buffers number))
+          (_ (error "The `%s' in `dired-preview-kill-buffers-method' is unknown" method))))))))
 
 (defun dired-preview--kill-large-buffers ()
   "Kill buffers previewing large files."
-  (mapc (lambda (pair)
-          (let ((buffer (cdr pair)))
-            (and (bufferp buffer)
-                 (kill-buffer buffer))))
-        dired-preview--large-files-alist)
+  (dolist (pair dired-preview--large-files-alist)
+    (let ((buffer (cdr pair)))
+      (and (bufferp buffer)
+           (kill-buffer buffer))))
   (setq dired-preview--large-files-alist nil))
 
 (defun dired-preview--kill-placeholder-buffers ()
@@ -210,30 +283,29 @@ until it drops below this number.")
 
 (defun dired-preview--delete-windows ()
   "Delete preview windows."
-  (mapc
-   (lambda (window)
-     (unless (or (one-window-p)
-                 (eq window (minibuffer-window)))
-       (delete-window window)))
-   (dired-preview--get-windows)))
+  (dolist (window (dired-preview--get-windows))
+    (when (and (not (one-window-p))
+               (window-live-p window)
+               (not (eq window (minibuffer-window))))
+      (delete-window window))))
 
 (defun dired-preview--file-ignored-p (file)
   "Return non-nil if FILE extension is among the ignored extensions.
 See user option `dired-preview-ignored-extensions-regexp'."
-  (when-let (((not (file-directory-p file)))
-             ((stringp dired-preview-ignored-extensions-regexp))
-             (ext (file-name-extension file :include-dot))
-             ((not (string-blank-p ext))))
+  (when-let* (((not (file-directory-p file)))
+              ((stringp dired-preview-ignored-extensions-regexp))
+              (ext (file-name-extension file :include-dot))
+              ((not (string-blank-p ext))))
     (string-match-p ext dired-preview-ignored-extensions-regexp)))
 
 (defun dired-preview--file-large-p (file)
   "Return non-nil if FILE exceeds `dired-preview-max-size'."
-  (> (file-attribute-size (file-attributes file)) dired-preview-max-size))
+  (>= (file-attribute-size (file-attributes file)) dired-preview-max-size))
 
 (defun dired-preview--file-displayed-p (file)
   "Return non-nil if FILE is already displayed in a window."
-  (when-let ((buffer (get-file-buffer file))
-             (window (get-buffer-window buffer)))
+  (when-let* ((buffer (get-file-buffer file))
+              (window (get-buffer-window buffer)))
     (window-live-p window)))
 
 (defun dired-preview--set-window-parameters (window value)
@@ -245,10 +317,14 @@ See user option `dired-preview-ignored-extensions-regexp'."
 
 (defun dired-preview--clean-up-window ()
   "Delete or clean up preview window."
-  (if (window-parameter (selected-window) 'dired-preview-window)
-      (dired-preview--delete-windows)
-    (dired-preview--set-window-parameters (selected-window) nil)
-    (remove-hook 'post-command-hook #'dired-preview--clean-up-window :local)))
+  (let* ((window (selected-window))
+         (buffer (window-buffer window)))
+    (if (window-parameter window 'dired-preview-window)
+        (dired-preview--delete-windows)
+      (dired-preview--rename-buffer (window-buffer window) :make-public)
+      (setq dired-preview--buffers (delq buffer dired-preview--buffers))
+      (dired-preview--set-window-parameters window nil)
+      (remove-hook 'post-command-hook #'dired-preview--clean-up-window :local))))
 
 ;; TODO 2024-04-22: Add PDF type and concomitant method to display its buffer.
 (defun dired-preview--infer-type (file)
@@ -272,6 +348,17 @@ FILE."
      (t
       (cons 'text file)))))
 
+(defmacro dired-preview-with-file-setup (&rest body)
+  "Run BODY while setting up the right preview environment."
+  (declare (indent 0))
+  `(cl-letf (((symbol-function 'recentf-track-opened-file) #'ignore))
+     (let ((file (cdr file))
+           (inhibit-message t)
+           (enable-dir-local-variables nil)
+           (enable-local-variables :safe)
+           (non-essential t))
+       ,@body)))
+
 (cl-defgeneric dired-preview--get-buffer (file)
   "Get a buffer for FILE.")
 
@@ -280,17 +367,13 @@ FILE."
 ;; that returns the method with its implementation?
 (cl-defmethod dired-preview--get-buffer ((file (head text)))
   "Get preview buffer for text FILE type."
-  (cl-letf (((symbol-function 'recentf-track-opened-file) #'ignore))
-    (let ((file (cdr file))
-          (inhibit-message t)
-          (enable-dir-local-variables nil)
-          (enable-local-variables :safe)
-          (non-essential t))
-      (find-file-noselect file :nowarn))))
+  (dired-preview-with-file-setup
+   (find-file-noselect file :nowarn)))
 
 (defun dired-preview--add-truncation-message ()
   "Add a message indicating that the previewed file is truncated."
-  (let ((end-ov (make-overlay (1- (point-max)) (point-max))))
+  (let* ((max (point-max))
+         (end-ov (make-overlay (1- max) max)))
     (overlay-put
      end-ov 'display
      (propertize "\n--PREVIEW TRUNCATED--" 'face 'shadow))))
@@ -325,17 +408,17 @@ Also see `dired-preview-open-dwim'."
 ;; NOTE 2024-07-29: Adapted from the `dired-do-open' found in Emacs 31.0.50.
 (defun dired-preview--open-externally (file)
   "Run appropriate command to open FILE externally."
-  (if-let ((command (cond
-                     ((executable-find "xdg-open")
-                      "xdg-open")
-                     ((memq system-type '(gnu/linux darwin))
-                      "open")
-                     ((memq system-type '(windows-nt ms-dos))
-                      "start")
-                     ((eq system-type 'cygwin)
-                      "cygstart")
-                     ((executable-find "run-mailcap")
-                      "run-mailcap"))))
+  (if-let* ((command (cond
+                      ((executable-find "xdg-open")
+                       "xdg-open")
+                      ((memq system-type '(gnu/linux darwin))
+                       "open")
+                      ((memq system-type '(windows-nt ms-dos))
+                       "start")
+                      ((eq system-type 'cygwin)
+                       "cygstart")
+                      ((executable-find "run-mailcap")
+                       "run-mailcap"))))
       (cond
        ((memq system-type '(gnu/linux))
         (call-process command nil 0 nil file))
@@ -362,7 +445,7 @@ Also see `dired-preview-find-file'."
   (interactive)
   (let (buffer)
     (dired-preview-with-window
-      (when-let ((file buffer-file-name))
+      (when-let* ((file buffer-file-name))
         (cond
          ((or (string-match-p dired-preview-media-extensions-regexp file)
               (string-match-p dired-preview-ignored-extensions-regexp file)
@@ -394,50 +477,44 @@ This technically runs `scroll-down-command'."
 
 (declare-function hexl-mode "hexl")
 (declare-function hexl-mode-exit "hexl" (&optional arg))
+(defvar hexl-follow-ascii)
 
 (defun dired-preview-hexl-toggle ()
   "Toggle preview between text and `hexl-mode'."
   (interactive)
   (dired-preview-with-window
-    (if (eq major-mode 'hexl-mode)
-        (hexl-mode-exit)
-      (hexl-mode)
-      (dired-preview--add-truncation-message))))
+    (let ((hexl-follow-ascii nil))
+      (if (eq major-mode 'hexl-mode)
+          (hexl-mode-exit)
+        (hexl-mode 1)))
+    (dired-preview--add-truncation-message)))
 
 (cl-defmethod dired-preview--get-buffer ((file (head large)))
   "Get preview buffer for large FILE.
 The size of the leading chunk is specified by
 `dired-preview-chunk-size'."
-  (cl-letf (((symbol-function 'recentf-track-opened-file) #'ignore))
-    (let ((file (cdr file))
-          (inhibit-message t)
-          (enable-dir-local-variables nil)
-          (enable-local-variables :safe)
-          (non-essential t))
-      (if-let* ((buffer (or (get-file-buffer file)
-                            (find-buffer-visiting file)
-                            (alist-get file dired-preview--large-files-alist
-                                       nil nil #'equal))))
-          buffer ; Buffer is already being visited, we can reuse it
-        (with-current-buffer (create-file-buffer file)
-          ;; We create a buffer with a partial preview
-          (buffer-disable-undo)
-          (insert-file-contents file nil 1 dired-preview-chunk-size 'replace)
-          (when (eq buffer-file-coding-system 'no-conversion)
-            (hexl-mode))
-          (dired-preview--add-truncation-message)
-          (read-only-mode t)
-          ;; Because this buffer is not marked as visiting FILE, we need to keep
-          ;; track of it ourselves.
-          (setf (alist-get file dired-preview--large-files-alist
-                           nil nil 'equal)
-                (current-buffer)))))))
+  (dired-preview-with-file-setup
+   (if-let* ((buffer (or (get-file-buffer file)
+                         (find-buffer-visiting file)
+                         (alist-get file dired-preview--large-files-alist nil nil #'equal))))
+       buffer ; Buffer is already being visited, we can reuse it
+     (with-current-buffer (create-file-buffer file)
+       ;; We create a buffer with a partial preview
+       (buffer-disable-undo)
+       (insert-file-contents file nil 1 dired-preview-chunk-size 'replace)
+       (when (eq buffer-file-coding-system 'no-conversion)
+         (let ((hexl-follow-ascii nil))
+           (hexl-mode 1)))
+       (dired-preview--add-truncation-message)
+       (read-only-mode t)
+       ;; Because this buffer is not marked as visiting FILE, we need to keep
+       ;; track of it ourselves.
+       (setf (alist-get file dired-preview--large-files-alist nil nil 'equal) (current-buffer))))))
 
 (cl-defmethod dired-preview--get-buffer ((file (head ignore)))
   "Get preview placeholder buffer for an ignored FILE."
   (let* ((file (cdr file))
-         (buffer-name (format "%s (no preview)"
-                              (file-name-nondirectory file))))
+         (buffer-name (format "%s (no preview)" (file-name-nondirectory file))))
     (or (get-buffer buffer-name)
         (with-current-buffer (get-buffer-create buffer-name)
           (set-visited-file-name file t)
@@ -449,39 +526,33 @@ The size of the leading chunk is specified by
 
 (cl-defmethod dired-preview--get-buffer ((file (head directory)))
   "Get preview buffer for directory FILE type."
-  (cl-letf (((symbol-function 'recentf-track-opened-file) #'ignore))
-    (let ((file (cdr file))
-          (inhibit-message t)
-          (enable-dir-local-variables nil)
-          (enable-local-variables :safe)
-          (non-essential t))
-      (dired-noselect file))))
+  (dired-preview-with-file-setup
+   (dired-noselect file)))
 
 ;; FIXME 2024-04-22: Best way to preview images and PDF files?  For now
 ;; this is the same as the text file type, though we need to refine
 ;; it.
 (cl-defmethod dired-preview--get-buffer ((file (head image)))
   "Get preview buffer for image FILE type."
-  (cl-letf (((symbol-function 'recentf-track-opened-file) #'ignore))
-    (let ((file (cdr file))
-          (inhibit-message t)
-          (enable-dir-local-variables nil)
-          (enable-local-variables :safe)
-          (non-essential t))
-      (find-file-noselect file :nowarn))))
+  (dired-preview-with-file-setup
+   (find-file-noselect file :nowarn)))
 
 (defun dired-preview--add-to-previews (file)
   "Add FILE to `dired-preview--buffers', if not already in a buffer.
-Always return FILE buffer."
+Return FILE buffer or nil."
   (cl-letf (((symbol-function 'recentf-track-opened-file) #'ignore))
-    (let ((buffer (find-buffer-visiting file)))
-      (if (buffer-live-p buffer)
-          buffer
-        (setq buffer (dired-preview--get-buffer (dired-preview--infer-type file))))
-      (with-current-buffer buffer
-        (add-hook 'post-command-hook #'dired-preview--clean-up-window nil :local))
-      (add-to-list 'dired-preview--buffers buffer)
-      buffer)))
+    (let* ((inhibit-message t)
+           (existing-buffer (or (find-buffer-visiting file)
+                                (and (file-directory-p file)
+                                     (car (dired-buffers-for-dir file)))))
+           (new-buffer (unless existing-buffer
+                         (dired-preview--get-buffer (dired-preview--infer-type file)))))
+      (dired-preview--kill-buffers)
+      (when new-buffer
+        (with-current-buffer new-buffer
+          (add-hook 'post-command-hook #'dired-preview--clean-up-window nil :local))
+        (add-to-list 'dired-preview--buffers new-buffer))
+      (or existing-buffer new-buffer))))
 
 (defun dired-preview--get-preview-buffer (file)
   "Return buffer to preview FILE in."
@@ -496,23 +567,27 @@ DIMENSION is either a `:width' or `:height' keyword.  It is
 checked against `split-width-threshold' or
 `split-height-threshold'"
   (pcase dimension
-    (:width (if-let ((window-width (floor (window-total-width) 2))
-                     ((> window-width fill-column)))
+    (:width (if-let* ((window-width (floor (window-total-width) 2))
+                      ((>= window-width fill-column)))
                 window-width
               fill-column))
     (:height (floor (window-height) 2))))
 
 (defun dired-preview-display-action-side ()
   "Pick a side window that is appropriate for the given frame."
-  (if-let (split-width-threshold
-           (width (window-body-width))
-           ((>= width (window-body-height)))
-           ((>= width split-width-threshold)))
+  (if-let* (split-width-threshold
+            (width (window-body-width))
+            ((>= width (window-body-height)))
+            ((>= width split-width-threshold)))
       `(:side right :dimension window-width :size ,(dired-preview-get-window-size :width))
     `(:side bottom :dimension window-height :size ,(dired-preview-get-window-size :height))))
 
 (defun dired-preview-display-action-alist-dwim ()
   "Reference function for `dired-preview-display-action-alist'.
+Determine whether to show a preview window on the right side or at the
+bottom of the frame, depending on the available space, and set the size
+of the window accordingly.
+
 Return a `display-buffer' action alist, as described in the
 aforementioned user option."
   (let ((properties (dired-preview-display-action-side)))
@@ -520,6 +595,19 @@ aforementioned user option."
       (side . ,(plist-get properties :side))
       (,(plist-get properties :dimension) . ,(plist-get properties :size)))))
 
+(defun dired-preview-display-action-alist-below ()
+  "Reference function for `dired-preview-display-action-alist'.
+Always show the preview window below the currently selected window and
+always keep the height of the preview window to 0.3 times that of the
+total height of the frame.
+
+Return a `display-buffer' action alist, as described in the
+aforementioned user option."
+  `((display-buffer-below-selected)
+    (window-height . 0.3)
+    (dedicated . t)
+    (preserve-size . (t . t))))
+
 (defvar dired-preview-trigger-commands
   '( dired-next-line
      dired-previous-line
@@ -546,13 +634,14 @@ aforementioned user option."
   "Kill preview buffers and delete their windows."
   (dired-preview--cancel-timer)
   (dired-preview--delete-windows)
-  (dired-preview--kill-buffers)
+  (dired-preview--kill-buffers :kill-all)
   (dired-preview--kill-large-buffers)
   (dired-preview--kill-placeholder-buffers))
 
 (defun dired-preview--close-previews-outside-dired ()
-  "Call `dired-preview--close-previews' if the current buffer is not in Dired mode."
-  (unless (eq major-mode 'dired-mode)
+  "Call `dired-preview--close-previews' if the current buffer is not in Dired mode.
+Do not consider the minibuffer as being another mode."
+  (unless (or (eq major-mode 'dired-mode) (minibufferp))
     (dired-preview--close-previews)
     (remove-hook 'window-state-change-hook #'dired-preview--close-previews-outside-dired)
     (put 'dired-preview-start 'function-executed nil)))
@@ -563,22 +652,51 @@ Only do it with the current major mode is Dired."
   (let ((action-alist (cond
                        ((functionp dired-preview-display-action-alist)
                         (funcall dired-preview-display-action-alist))
-                       (dired-preview-display-action-alist)
-                       (t
-                        (dired-preview-display-action-alist-dwim)))))
+                       (dired-preview-display-action-alist))))
     (display-buffer buffer action-alist)))
 
+(defun dired-preview--remove-preview-indicator (name)
+  "Remove `dired-preview-buffer-name-indicator' from NAME."
+  (string-trim
+   (string-replace dired-preview-buffer-name-indicator "" name)
+   "[\s\t\n\r]+" nil))
+
+(defun dired-preview--rename-buffer (buffer &optional make-public)
+  "Rename BUFFER to have `dired-preview-buffer-name-indicator'.
+With optional MAKE-PUBLIC, remove the indicator."
+  (let ((name (buffer-name buffer)))
+    (with-current-buffer buffer
+      (cond
+       (make-public
+        (rename-buffer (dired-preview--remove-preview-indicator name) :make-unique))
+       ((and (not (string-match-p (regexp-quote dired-preview-buffer-name-indicator) name))
+             (memq buffer dired-preview--buffers))
+        (rename-buffer (format "%s %s" dired-preview-buffer-name-indicator name) :make-unique))))))
+
 (defun dired-preview-display-file (file)
   "Display preview of FILE if appropriate."
   (dired-preview--delete-windows)
-  (when-let ((buffer (dired-preview--get-preview-buffer file)))
+  (when-let* ((buffer (dired-preview--get-preview-buffer file)))
     (dired-preview--display-buffer buffer)
-    (when-let ((window (get-buffer-window buffer)))
+    (dired-preview--rename-buffer buffer)
+    (when-let* ((window (get-buffer-window buffer)))
       (dired-preview--set-window-parameters window t))))
 
+(defvar dired-preview-encryption-file-extensions '(".gpg" ".age")
+  "List of strings specifying file extensions for encryption.")
+
+(defun dired-preview--file-encrypted-p (file)
+  "Return non-nil if FILE is encrypted.
+More specifically, test if FILE has an extension among the
+`dired-preview-encryption-file-extensions'."
+  (when-let* ((extension (file-name-extension file :include-period)))
+    (member extension dired-preview-encryption-file-extensions)))
+
 (defun dired-preview--preview-p (file)
   "Return non-nil if FILE can be previewed."
   (and file
+       (not (string-match-p "/\\./" file))
+       (not (dired-preview--file-encrypted-p file))
        (or (file-regular-p file) (file-directory-p file))
        (file-readable-p file)
        (not (dired-preview--file-displayed-p file))
@@ -595,22 +713,26 @@ Only do it with the current major mode is Dired."
   "Trigger display of file at point after `dired-preview-trigger-commands'.
 With optional NO-DELAY do not start a timer.  Otherwise produce
 the preview with `dired-preview-delay' of idleness."
-  (add-hook 'window-state-change-hook #'dired-preview--close-previews-outside-dired)
-  (dired-preview--cancel-timer)
-  (let* ((file (dired-file-name-at-point))
-         (preview (dired-preview--preview-p file)))
-    (cond
-     ((and preview (memq this-command dired-preview-trigger-commands))
-      (if no-delay
-          (dired-preview-display-file file)
-        (setq dired-preview--timer
-              (run-with-idle-timer dired-preview-delay nil #'dired-preview-display-file file))))
-     ((and file preview)
-      (dired-preview-start file))
-     ((and (not preview)
-           (memq this-command dired-preview-trigger-commands))
-      (dired-preview--delete-windows)))
-    (dired-preview--close-previews-outside-dired)))
+  (condition-case nil
+      (if (eq major-mode 'dired-mode)
+          (progn
+            (add-hook 'window-state-change-hook #'dired-preview--close-previews-outside-dired)
+            (dired-preview--cancel-timer)
+            (let* ((file (dired-file-name-at-point))
+                   (preview (dired-preview--preview-p file)))
+              (cond
+               ((and preview (memq this-command dired-preview-trigger-commands))
+                (if no-delay
+                    (dired-preview-display-file file)
+                  (setq dired-preview--timer
+                        (run-with-idle-timer dired-preview-delay nil #'dired-preview-display-file file))))
+               (preview
+                (dired-preview-start file))
+               ((and (not preview)
+                     (memq this-command dired-preview-trigger-commands))
+                (dired-preview--delete-windows)))))
+        (dired-preview--close-previews-outside-dired))
+    ((error user-error quit) nil)))
 
 (defun dired-preview-disable-preview ()
   "Disable Dired preview."
@@ -629,7 +751,7 @@ the preview with `dired-preview-delay' of idleness."
 
 (defvar dired-preview-mode-map
   (let ((map (make-sparse-keymap)))
-    (define-key map (kbd "C-c C-c") #'dired-preview-hexl-toggle)
+    (define-key map (kbd "C-c C-x") #'dired-preview-hexl-toggle)
     (define-key map (kbd "C-c C-f") #'dired-preview-find-file)
     (define-key map (kbd "C-c C-o") #'dired-preview-open-dwim)
     (define-key map (kbd "C-c C-u") #'dired-preview-page-up)
